#include "mainwindow.h"
#include "ui_mainwindow.h"


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);


    //init
    isDownloadStart = false;
    pd = NULL;

    //使用qextserilport扩展累类，详细请参考qextserilport例子。
    //枚举串口设备
    QList<QextPortInfo> ports = QextSerialEnumerator::getPorts();
    qDebug() << "List of ports:";

    for (int i = 0; i < ports.size(); i++) {
        //
        ui->portName_comboBox->addItem(ports.at(i).portName,0);

        qDebug() << "port name:" << ports.at(i).portName;
        qDebug() << "friendly name:" << ports.at(i).friendName;
        qDebug() << "physical name:" << ports.at(i).physName;
        qDebug() << "enumerator name:" << ports.at(i).enumName;
        qDebug() << "vendor ID:" << QString::number(ports.at(i).vendorID, 16);
        qDebug() << "product ID:" << QString::number(ports.at(i).productID, 16);
        qDebug() << "===================================";

    }
    //建立消息机制
    connect(&serial,SIGNAL(lineReceived(QString)),
            this,SLOT(onLineReceived(QString)));




    //获取固件版本和闪存使用情况消息
    connect(this,SIGNAL(found_ver(QString)),this,SLOT(on_found_ver(QString)));
    connect(this,SIGNAL(found_mem(QString)),this,SLOT(on_found_mem(QString)));


    //下载gps数据消息
     connect(this,SIGNAL(found_connect()),this,SLOT(on_found_connect()));
     connect(this,SIGNAL(found_download(QString)),this,SLOT(on_found_download(QString)));
     connect(this,SIGNAL(found_downcomplete()),this,SLOT(on_found_downcomplete()));
     connect(this,SIGNAL(downloadstart(QString)),this,SLOT(on_downloadstart(QString)));

    //刷除闪存内的gps数据
     //#StartErase
     connect(this,SIGNAL(found_starterase()),this,SLOT(on_found_starterase()));
     //#erasecomplete
     connect(this,SIGNAL(found_erasecomplete()),this,SLOT(on_found_erasecomplete()));


    //升级固件消息处理
    connect(this,SIGNAL(upload_firmware_found_ok()),this,SLOT(on_upload_firmware_found_ok()));
    connect(this,SIGNAL(uplaod_firmware_found_error()),this,SLOT(on_upload_firmware_found_error()));
    connect(this,SIGNAL(upload_firmware_found_ready()),this,SLOT(on_upload_firmware_found_ready()));
    connect(this,SIGNAL(upload_firmware_found_finish()),this,SLOT(on_upload_firmware_found_finish()));
    connect(this,SIGNAL(upload_firmware_send_data(int)),this,SLOT(on_upload_firmware_send_data(int)));

}

MainWindow::~MainWindow()
{
    delete ui;
}



//线程安全的处理串口返回的字符串数据。
void MainWindow::onLineReceived(QString data)
{

    /*******************开始＃ver指令处理区域********************/
    /////////////////////////////////////////////////////////
    //处理＃ver指令返回的自字符串
    //received #ver from port  "#Ver: Solmeta PRO V10.1-02A24"
    //received #ver from port  "#mem:2531.00KB / 4096KB

    if ( data.startsWith(QString("#Ver:")))
    {
        emit found_ver(data);

        qDebug()  <<"received #ver from port " << data;
        return;
    }

    if ( data.startsWith(QString("#mem:")))
    {
        emit found_mem(data);

        qDebug()  <<"received #mem from port " << data;
        return;
    }
    /*****************结束＃ver指令处理区域*************/
    /////////////////////////////////////////////////


    /*******************开始＃down指令处理区域********************/
    /////////////////////////////////////////////////////////
    //处理＃down指令返回的自字符串
    //received #connect from port  "#connect
    //  收到＃connect指令后要在指定时间内完成切换串口波特率为115200就可以准备下载
    if(data.startsWith(QString("#connect")))
    {
        qDebug()  <<"received #connect from port " << data;
        emit found_connect();
        return;
    }

    if(isDownloadStart)
    {
        qDebug()  <<"begin download Gps Data from port ";

        emit downloadstart(data);
        return;
    }

    if(data.startsWith(QString("#download")))
    {
        qDebug()  <<"received #download from port " << data;
        emit found_download(data);
        return;

    }


    //完成文件写入操作
    if(data.contains(QString("#downcomplete"), Qt::CaseInsensitive))
    {
        qDebug()  <<"received #downloadcomplete from port " << data;
        emit found_downcomplete();
        return;
    }

    /*******************结束＃down指令处理区域********************/
    /////////////////////////////////////////////////////////


    /*******************开始＃erase指令处理区域 *****************/
    //////////////////////////////////////////////////////////
    if(data.startsWith(QString("#StartErase")))
    {

        emit found_starterase();
        return;
    }

    if(data.startsWith(QString("#erasecomplete")))
    {

        qDebug() << "erase memory complete!";
        emit found_erasecomplete();

        return;
    }
    /*******************结束＃erase指令处理区域 *****************/
    //////////////////////////////////////////////////////////

    /******************开始＃upgrade指令处理区域 *****************/
    //////////////////////////////////////////////////////////

    if(data.startsWith(QString("@lok")))
    {
        qDebug() <<data;

        emit upload_firmware_found_ok();

        return;
    }
    if(data.startsWith(QString("@lerror")))
    {
        qDebug() <<data;

        emit uplaod_firmware_found_error();

        return;
    }
    if(data.startsWith(QString("@LReady")))
    {
        qDebug() <<data;

        emit upload_firmware_found_ready();

        return;
    }
    if(data.startsWith(QString("@lcomplete")))
    {
        qDebug() <<data ;
        emit upload_firmware_found_finish();

        return;
    }
    /*******************结束＃upgrade指令处理区域 *****************/
    //////////////////////////////////////////////////////////

}

//设置gps数据下载路径
void MainWindow::on_download_path_pushButton_clicked()
{
    QString gps_data_dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                             "/home",
                                                             QFileDialog::ShowDirsOnly
                                                             | QFileDialog::DontResolveSymlinks);
    ui->lineEdit_Gps_Data_path->setText(gps_data_dir);

    qDebug()  <<"Gps Data Path set in " << gps_data_dir;
}


//开始下载gps数据
void MainWindow::on_download_pushButton_clicked()
{

    if( !init_serialport(0,4800))
    {
        return;
    }


    //如果端口已经打开
    if(serial.isOpen())
    {
        //检查gps数据目录的可访问性
        QDir dir;
        if(! dir.exists ( ui->lineEdit_Gps_Data_path->text() ))
        {
            qDebug()  <<"Gps Data Path no set ";

            QString dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                            "/home",
                                                            QFileDialog::ShowDirsOnly
                                                            | QFileDialog::DontResolveSymlinks);
            ui->lineEdit_Gps_Data_path->setText(dir);

            qDebug()  <<"Gps Data Path set in " << dir;
        }

        QFile savefile;
        QDate  qd = QDate::currentDate (); 
        qDebug() << "file path:" << ui->lineEdit_Gps_Data_path->text()  +QString("/Solmeta_").append(qd.toString("yyyyMMdd")+".txt")  ;
        if( savefile.exists ( ui->lineEdit_Gps_Data_path->text()  +QString("/Solmeta_").append(qd.toString("yyyyMMdd")+".txt")   ))
        {
            //跳出警框 存在相同文件名
            QMessageBox msgBox;
            QMessageBox::critical(this, tr("have same file name in the dir path"),
                                  tr("found same filename ,please select the other save file path."),
                                  QMessageBox::Abort);

            qDebug() << "have double file in same dir path!";

            return;
        }

        //开始下载数据
        //发送＃down 指令
        serial.write("#down\n");
        qDebug() << "write #down to port";



    }

}

//根据访问参数设置 打开串口
void MainWindow::on_open_Button_clicked()
{
    if(!serial.isOpen())
    {
        //设置访问设备名字和传输波特 率

        qDebug() << ui->portName_comboBox->currentText();


        serial.open(ui->portName_comboBox->currentText(), ui->baudrate_comboBox->currentText().toInt() );

        if(!serial.isOpen())
        {
            qDebug() << "open port faild !!";

            //跳出警框
            QMessageBox msgBox;
            QMessageBox::critical(this, tr("open port faild!"),
                                  tr("Please check the usb cable."),
                                  QMessageBox::Abort);
            return;
        }

        //修改打开的按钮ui状态
        ui->open_Button->setEnabled(false);

        qDebug() << "port in open at speed: "<<  ui->baudrate_comboBox->currentText() ;
    }
}


//查询设备固件版本和flash芯片使用情况。
void MainWindow::on_checkinfo_pushButton__clicked()
{

    if(!init_serialport(0,4800))
    {
        return;
    }

    ui->version_label->setText(tr("empty"));
    ui->flashmemory_label->setText(tr("empty"));


    //往单片机发送  #ver 指令
    //单片机器回复 ：
    //#Ver: Solmeta PRO V10.1-02A24
    //#mem:2531.00KB / 4096KB

    //发送＃ver指令
    serial.write("#ver\n");
    qDebug() << "write #ver to port";

}


//关闭端口
void MainWindow::on_stop_Button_clicked()
{
    if(serial.isOpen())
    {
        serial.close();
        ui->open_Button->setEnabled(true);
        qDebug() << "port in close";
    }
}

//刷除存储芯片上的数据
void MainWindow::on_erase_memory_pushButton_clicked()
{
    if(!init_serialport(0,4800))
    {
        return;
    }

    //重新再发一次  #erase指令
    serial.write(QString("#erase\n"));
    qDebug() << "write #erase port in open at speed: "<<  ui->baudrate_comboBox->currentText() ;

}


//设置升级固件文件名
void MainWindow::on_firmware_upgrade_path_pushButton_clicked()
{
    QString firmware_upgarde_fileName = QFileDialog::getOpenFileName(this, QString("Open File"),
                                                                     "/home",
                                                                     QString("Images (*.bin *.hex)"));

    ui->lineEdit_firmware_upgrade_path->setText(firmware_upgarde_fileName);

    qDebug()  <<"firmware upgrade filename Path set in " << firmware_upgarde_fileName;


}



//开始升级
void MainWindow::on_firmware_upgrade_pushButton_clicked()
{
     if(!init_serialport(1,115200))
    {
         return;
    }


    if (! QFile::exists(ui->lineEdit_firmware_upgrade_path->text()))
    {
        //跳出警框
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("open firmware file faild!"),
                              tr("open firmware file faild! Please check the firmware file path."),
                              QMessageBox::Abort);
        qDebug() << "open firmware file faild !!";
        return;
    }

    qDebug() << "write  port in open at speed: "<<  ui->baudrate_comboBox->currentText() ;

    //if (m_firmware_file){delete m_firmware_file; m_firmware_file = NULL;}else{qDebug() << "m_firmware_file is null!!";}
    m_firmware_file = new QFile(ui->lineEdit_firmware_upgrade_path->text());

    if (!m_firmware_file->open(QIODevice::ReadOnly))
    {

        //跳出警框
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("open firmware file faild!"),
                              tr("open firmware file faild! Please check the firmware file path."),
                              QMessageBox::Abort);
        qDebug() << "open firmware file faild !!";
        return;
    }

    if(m_firmware_file->size()<=0)
    {
        //跳出警框
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("firmware file size!"),
                              tr("check firmware file size!"),
                              QMessageBox::Abort);
        qDebug() << "check firmware file size!";
        return;
    }



    if (pd){delete pd; pd = NULL;}else{qDebug() << "pd is null!!";}
    pd = new QProgressDialog(tr("upgrading firmware  operation in progress."), tr("OK"), 0, m_firmware_file->size());


    QPushButton *button = new QPushButton(tr("&Upgrading"), this);
    button->setEnabled(false);

    pd->setCancelButton(button);

    fileLen = 0;
    steps= 0;

    ReadFileLine(m_TxBuf,&fileLen);


    serial.write(QString(m_TxBuf));

    qDebug() << "write port:" << QString(m_TxBuf).replace(QString("\r\n"),QString("")) ;

    emit on_upload_firmware_send_data(fileLen);

}

//重新打开串口
bool MainWindow::init_serialport(int ui_baud_rate_item , int baud_rate)
{
    /*
    if(serial.isOpen())
    {
        serial.close();
    }
    */
    //重新打开串口
    ui->baudrate_comboBox->setCurrentIndex(ui_baud_rate_item);
    serial.open(ui->portName_comboBox->currentText(), baud_rate );
    if(!serial.isOpen() )
    {
        qDebug() << "open port faild !!";

        //跳出警框
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("open port faild!"),
                              tr("Please check the usb cable."),
                              QMessageBox::Abort);

        return false;
    }

    ui->open_Button->setEnabled(false);

    qDebug() << "open port finish !!";
    return true;
}

//转换bin‘为acsii hex
bool  MainWindow::ReadFileLine(char *buf,int *len)
{
    bool bresult	= true;
    char ctemp	= 0;
    int i=0;
    while(1)
    {
        if(m_firmware_file->read( &ctemp,1))
        {
            ctemp	-= 0x4E;
            *(buf+i)	= ctemp;
            i++;
            if(ctemp=='\n')
                break;
        }
        else
        {
            bresult		= FALSE;
            break;
        }
    }
    *len		= i;
    *(buf+i)	= 0;

    return bresult;
}

//下位机器返回ok命令，pc端向下位机发送新的hex升级固件代码
void MainWindow::on_upload_firmware_found_ok()
{
    ReadFileLine(m_TxBuf,&fileLen);


    serial.write(QString(m_TxBuf));

    qDebug() << "write port:" <<  QString(m_TxBuf).replace(QString("\r\n"),QString("")) ;

    emit on_upload_firmware_send_data(fileLen);
}

//下位机器返回error命令，pc端做 异常处理
void MainWindow::on_upload_firmware_found_error()
{
    //跳出警框
    QMessageBox msgBox;
    QMessageBox::critical(this, tr("open port faild!"),
                          tr("Upgrade Faild."),
                          QMessageBox::Abort);

    if (pd){delete pd; pd = NULL;qDebug() << "pd set null!!";}else{qDebug() << "pd is null!!";}

    return;
}


//下位机器返回ready命令，pc端向下位机发送新的hex升级固件代码
void MainWindow::on_upload_firmware_found_ready()
{
    ReadFileLine(m_TxBuf,&fileLen);


    serial.write(QString(m_TxBuf));

    //调整进度条
    emit on_upload_firmware_send_data(fileLen);
}


//升级固件完成
void MainWindow::on_upload_firmware_found_finish()
{

    //跳出警框
    QMessageBox msgBox;
    QMessageBox::critical(this, tr("Upgrade Finish."),
                          tr("Upgrade Finish."),
                          QMessageBox::Ok);

    if (steps > pd->maximum())
    {
        if (pd){delete pd; pd = NULL;qDebug() << "pd set null!!";}else{qDebug() << "pd is null!!";}
    }//if

}

//修改升级固件的进度跳
void MainWindow::on_upload_firmware_send_data(int value)
{
    steps = steps + value;
    pd->setValue(steps);

}


void MainWindow::on_found_ver(QString data)
{
    ui->version_label->setText(data.mid( QString("#Ver:").length(),
                                         data.length()-QString("#Ver:").length() )
                               );
}



void MainWindow::on_found_mem(QString data)
{
    ui->flashmemory_label->setText(tr("free:")+
                                   data.mid( QString("#mem:").length(),
                                             data.length()-QString("#mem:").length() ) +
                                   tr(" total")
                                   );
}

void MainWindow::on_found_connect()
{
    sleep(1);

    if(! init_serialport(1,115200) )
    {
        return;
    }

    //重新再发一次  #down指令
    serial.write("#down\n");
    qDebug() << "write #down to port in open at 115200";


    if (pd){delete pd; pd = NULL;}else{qDebug() << "pd is null!!";}
    pd = new QProgressDialog(tr("Download data operation in progress."), tr("OK"), 0, 100);
    steps = 0;

    QPushButton *button = new QPushButton(tr("&Downloading"), this);
    button->setEnabled(false);

    pd->setCancelButton(button);

}

void MainWindow::on_found_download(QString data)
{
    data.replace(QString("#download"), QString(""));
    bool ok = false;

    //255 = hex FF
    download_size = data.toInt(&ok , 16)*255;

    pd->setMaximum(download_size);
    steps = 0;
    //set Download Start Flag = true;
    isDownloadStart = true;
    gpsdata.clear();
}

void MainWindow::on_downloadstart(QString data)
{
    if (download_size > 0)
    {
        gpsdata << data <<"\r\n";

        //... perform one percent of the operation
        steps = steps + data.length()+QString("\r\n").length();
        pd->setValue(steps);


        if (steps > pd->maximum())
        {
            if (pd){delete pd; pd = NULL;qDebug() << "pd set null!!";}else{qDebug() << "pd is null!!";}
            isDownloadStart = 0;
        }//if

    }
    else
    {
        //跳出警框 下载文件大小为零
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("No Data !"),
                              tr("Please check the flash memory!."),
                              QMessageBox::Abort);

        qDebug() << "No Data !";

        if (pd){delete pd; pd = NULL;qDebug() << "pd set null!!";}else{qDebug() << "pd is null!!";}



        gpsdata.clear();
        isDownloadStart = 0;

    }
}

void MainWindow::on_found_downcomplete()
{
    gpsdata.replaceInStrings("#downcomplete", "\r\n");

    QDate  qd = QDate::currentDate ();
    qDebug() << ui->lineEdit_Gps_Data_path->text()  +QString("/Solmeta_").append(qd.toString("yyyyMMdd")+".txt");
    QFile savedata(ui->lineEdit_Gps_Data_path->text()  +QString("/Solmeta_").append(qd.toString("yyyyMMdd")+".txt"));

    if (savedata.open(QFile::WriteOnly | QFile::Truncate)) {
        QTextStream out(&savedata);

        for(int i =0 ; i<gpsdata.count();i++)
            out << gpsdata.value(i) ;

        //收尾操作
        if (pd){delete pd; pd = NULL;qDebug() << "pd not null now set null!!";}else{qDebug() << "pd is null!!";}

        gpsdata.clear();
        isDownloadStart = 0;

        qDebug() << "save Gps Data finish!";

        //跳出警框 保存文件失败
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("save Gps Data finish! "),
                              tr(" save Gps Data finish! "),
                              QMessageBox::Ok);

    }else
    {
        //跳出警框 保存文件失败
        QMessageBox msgBox;
        QMessageBox::critical(this, tr("save Gps Data faild! "),
                              tr(" Save file faild ! please check file permissions ."),
                              QMessageBox::Abort);

        qDebug() << "save Gps Data faild!";
    }

}


void MainWindow::on_found_starterase()
{

    if (pd){delete pd; pd = NULL;qDebug() << "pd not null now set null!!";}else{qDebug() << "pd is null!!";}
    pd = new QProgressDialog(tr("Erasing memory..."), "OK", 0, 100);
    pd->setValue(50);

}

void MainWindow::on_found_erasecomplete( )
{
    pd->setValue(100);

    //跳出警框 保存文件失败
    QMessageBox msgBox;
    QMessageBox::information(this, 0,
                             tr("erase memory complete!"),
                             QMessageBox::Ok);

}


//切换多国语言
void MainWindow::on_language_comboBox_currentIndexChanged(int index)
{
   static QTranslator *translator_lang;

   if(translator_lang)
   {
       qApp->removeTranslator(translator_lang);
       delete translator_lang;
       translator_lang =NULL;
   }

   translator_lang = new QTranslator();

    switch (index){
    case 0:
        if( translator_lang->load(QCoreApplication::applicationDirPath() + QString("/../language/Multi_eng")) == false)
        {
            QMessageBox msgBox;
            msgBox.setText(QCoreApplication::applicationDirPath());
            msgBox.exec();
        }
        break;
    case 1:
        if( translator_lang->load(QCoreApplication::applicationDirPath() + QString("/../language/Multi_fr"),".") == false )
        {
            QMessageBox msgBox;
            msgBox.setText("The document has been modified.");
            msgBox.exec();
        }
        break;
    case 2:
        if( translator_lang->load(QCoreApplication::applicationDirPath() + QString("/../language/Multi_Ger")) == false)
        {
            QMessageBox msgBox;
            msgBox.setText("The document has been modified.");
            msgBox.exec();
        }
        break;
    }

    qApp->installTranslator(translator_lang);


    //ui->retranslateUi(this)
    ui->groupBox_2->setTitle(QApplication::translate("MainWindow", "Language:", 0, QApplication::UnicodeUTF8));
    ui->label_version->setText(QApplication::translate("MainWindow", "Version:", 0, QApplication::UnicodeUTF8));
    ui->label_flash_memory->setText(QApplication::translate("MainWindow", "Flash Memory:", 0, QApplication::UnicodeUTF8));
    ui->checkinfo_pushButton_->setText(QApplication::translate("MainWindow", "Check Info", 0, QApplication::UnicodeUTF8));
    ui->groupBox->setTitle(QApplication::translate("MainWindow", "Port Configuration:", 0, QApplication::UnicodeUTF8));
    ui->label_port->setText(QApplication::translate("MainWindow", "Port:", 0, QApplication::UnicodeUTF8));
    ui->open_Button->setText(QApplication::translate("MainWindow", "Open", 0, QApplication::UnicodeUTF8));
    ui->label_baud_rate->setText(QApplication::translate("MainWindow", "Baud Rate:", 0, QApplication::UnicodeUTF8));
    ui->stop_Button->setText(QApplication::translate("MainWindow", "Stop", 0, QApplication::UnicodeUTF8));
    ui->groupBox_4->setTitle(QApplication::translate("MainWindow", "Download Gps Data:", 0, QApplication::UnicodeUTF8));
    ui->label_Gps_Data_path->setText(QApplication::translate("MainWindow", "Gps Data Path:", 0, QApplication::UnicodeUTF8));
    ui->download_path_pushButton->setText(QApplication::translate("MainWindow", "Browse...", 0, QApplication::UnicodeUTF8));
    ui->download_pushButton->setText(QApplication::translate("MainWindow", "Download Data", 0, QApplication::UnicodeUTF8));
    ui->erase_memory_pushButton->setText(QApplication::translate("MainWindow", "Erase Memory", 0, QApplication::UnicodeUTF8));
    ui->groupBox_5->setTitle(QApplication::translate("MainWindow", "Firmware Upgrade:", 0, QApplication::UnicodeUTF8));
    ui->label_firmware_upgrade_warning->setText(QApplication::translate("MainWindow", "[Warning]: Updating firmware is a dangerous operation.Please confirm whether the file format of firmware is correct before updating. And make sure do not unplug the connecting cable between the device and computer during the period of downloading.", 0, QApplication::UnicodeUTF8));
    ui->label_fireware_upgrade->setText(QApplication::translate("MainWindow", "Firmware Path:", 0, QApplication::UnicodeUTF8));
    ui->firmware_upgrade_path_pushButton->setText(QApplication::translate("MainWindow", "Browse...", 0, QApplication::UnicodeUTF8));
    ui->firmware_upgrade_pushButton->setText(QApplication::translate("MainWindow", "Download Update", 0, QApplication::UnicodeUTF8));

}

void MainWindow::on_actionSolmeta_Utility_triggered()
{
    QMessageBox msgBox;
    msgBox.setText("Copyright 2009-2010 Solmeta,Inc.");
    msgBox.exec();
}
